﻿using Org.BouncyCastle.Math;
using System;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using VGCA.MSS;

namespace MSSP_Sample
{
    public class MSSP_API : ISIM_API
    {
        String PKCS1 = "http://uri.etsi.org/TS102204/v1.1.2#PKCS1";
        String PKCS1_SHA256_PSS = "http://uri.etsi.org/TS102204/v1.1.2/sign/SHA-256/RSA#PKCS1_PSS";

        //APID can phu hop voi ip cua server cai dat dich vu
        String apId = "http://ap.mobile-id.vn/viettelca";
        String apPwd = "12345678";

        String msspId = "http://hmssp.mobile-id.vn";

        String msspSignatureUrl = "https://mpki1.ca.gov.vn:18083/soap/services/MSS_SignaturePort";

        String msspRegUrl = "https://mpki1.ca.gov.vn:18083/soap/services/MSS_RegistrationPort";

        X509Certificate2 apCertificate = null;
        public MSSP_API(String apId, String apPwd, String apP12, String apP12Pwd)
        {
            this.apId = apId;
            this.apPwd = apPwd;
            try
            {
                apCertificate = new X509Certificate2(apP12, apP12Pwd,
                    X509KeyStorageFlags.MachineKeySet |
                    X509KeyStorageFlags.PersistKeySet |
                    X509KeyStorageFlags.Exportable);
            }catch(Exception ex)
            {
                throw new ArgumentException("Chứng thư số AP hoặc mật khẩu không hợp lệ (Mã: 0x002)", ex);
            }
        }

        public MSSP_API(String apId, String apPwd, X509Certificate2 apCert)
        {
            this.apId = apId;
            this.apPwd = apPwd;
            this.apCertificate = apCert;
        }

        public virtual byte[] Sign(string phone, string messageToBeDisplayed, byte[] message)
        {
            String apTransId = "A" + DateTime.Now.ToString("yyMMddHH") + DateTime.Now.Minute / 15;
            String msisdn = phone;

            byte[] hashedstr = message;
            String dataToBeDisplayed = messageToBeDisplayed;
            String signatureProfile = "http://mobile-id.vn/digitalSignature";
            String mss_format = PKCS1;
            MessagingModeType messagingMode = MessagingModeType.synch;

            MSS_SignatureReqType sigReq = new MSS_SignatureReqType();

            sigReq.MajorVersion = BigInteger.ValueOf(1L).ToString();
            sigReq.MinorVersion = BigInteger.ValueOf(1L).ToString();

            MessageAbstractTypeAP_Info aiObject = new MessageAbstractTypeAP_Info();
            aiObject.AP_ID = apId;
            aiObject.AP_PWD = apPwd;

            aiObject.AP_TransID = apTransId;
            aiObject.Instant = DateTime.Now;
            sigReq.AP_Info = aiObject;

            MessageAbstractTypeMSSP_Info miObject = new MessageAbstractTypeMSSP_Info();
            MeshMemberType _msspId = new MeshMemberType();
            _msspId.URI = msspId;
            miObject.MSSP_ID = _msspId;
            sigReq.MSSP_Info = miObject;

            MobileUserType muObject = new MobileUserType();
            muObject.MSISDN = msisdn;
            sigReq.MobileUser = muObject;

            DataType dsObject = new DataType();
            dsObject.MimeType = "application/octet-stream";
            dsObject.Encoding = "base64";
            dsObject.Value = Convert.ToBase64String(hashedstr);
            sigReq.DataToBeSigned = dsObject;

            if (dataToBeDisplayed != null)
            {
                DataType ddObject = new DataType();
                ddObject.MimeType = "text/plain";
                ddObject.Encoding = "UTF-8";
                ddObject.Value = dataToBeDisplayed;

                sigReq.DataToBeDisplayed = ddObject;
            }

            mssURIType spObject = new mssURIType();
            spObject.mssURI = new Uri(signatureProfile).ToString();
            sigReq.SignatureProfile = spObject;

            if (mss_format != null)
            {
                mssURIType mfObject = new mssURIType();
                mfObject.mssURI = new Uri(mss_format).ToString();
                sigReq.MSS_Format = mfObject;
            }

            sigReq.MessagingMode = messagingMode;
            sigReq.AdditionalServices = new AdditionalServiceType[] {
                new AdditionalServiceType()
                {
                    Description = new mssURIType()
                    {
                        mssURI = "http://mobile-id.vn/MSSP/v1.0.0#signingCertificate"
                    }
                },
                new AdditionalServiceType()
                {
                    Description = new mssURIType()
                    {
                        mssURI = "http://mobile-id.vn/MSSP/v1.0.0#signatureValidation"
                    }
                }
            };
            sigReq.TimeOut = "300";

            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12 | SecurityProtocolType.Ssl3;
            ServicePointManager.Expect100Continue = false;
            ServicePointManager.ServerCertificateValidationCallback += new RemoteCertificateValidationCallback((sender, certificate, chain, policyErrors) => { return true; });
            
            MSS_SignatureBinding mssSignatureService = new MSS_SignatureBinding();
            mssSignatureService.Url = msspSignatureUrl;
            mssSignatureService.UseDefaultCredentials = true;
            mssSignatureService.ClientCertificates.Add(apCertificate);

            MSS_SignatureRespType sigResp = mssSignatureService.MSS_Signature(sigReq);

            long statusCode = long.Parse(sigResp.Status.StatusCode.Value);

            byte[] sigBuff = sigResp.MSS_Signature.Item;

            return sigBuff;
        }

        public virtual byte[] SignPSS(string phone, string messageToBeDisplayed, byte[] message)
        {
            String apTransId = "A" + DateTime.Now.ToString("yyMMddHH") + DateTime.Now.Minute / 15;
            String msisdn = phone;

            byte[] hashedstr = message;
            String dataToBeDisplayed = messageToBeDisplayed;
            String signatureProfile = "http://mobile-id.vn/digitalSignature";
            String mss_format = PKCS1_SHA256_PSS;
            MessagingModeType messagingMode = MessagingModeType.synch;

            MSS_SignatureReqType sigReq = new MSS_SignatureReqType();

            sigReq.MajorVersion = BigInteger.ValueOf(1L).ToString();
            sigReq.MinorVersion = BigInteger.ValueOf(1L).ToString();

            MessageAbstractTypeAP_Info aiObject = new MessageAbstractTypeAP_Info();
            aiObject.AP_ID = apId;
            aiObject.AP_PWD = apPwd;

            aiObject.AP_TransID = apTransId;
            aiObject.Instant = DateTime.Now;
            sigReq.AP_Info = aiObject;

            MessageAbstractTypeMSSP_Info miObject = new MessageAbstractTypeMSSP_Info();
            MeshMemberType _msspId = new MeshMemberType();
            _msspId.URI = msspId;
            miObject.MSSP_ID = _msspId;
            sigReq.MSSP_Info = miObject;

            MobileUserType muObject = new MobileUserType();
            muObject.MSISDN = msisdn;
            sigReq.MobileUser = muObject;

            DataType dsObject = new DataType();
            dsObject.MimeType = "application/octet-stream";
            dsObject.Encoding = "base64";
            dsObject.Value = Convert.ToBase64String(hashedstr);
            sigReq.DataToBeSigned = dsObject;

            if (dataToBeDisplayed != null)
            {
                DataType ddObject = new DataType();
                ddObject.MimeType = "text/plain";
                ddObject.Encoding = "UTF-8";
                ddObject.Value = dataToBeDisplayed;

                sigReq.DataToBeDisplayed = ddObject;
            }

            mssURIType spObject = new mssURIType();
            spObject.mssURI = new Uri(signatureProfile).ToString();
            sigReq.SignatureProfile = spObject;

            if (mss_format != null)
            {
                mssURIType mfObject = new mssURIType();
                mfObject.mssURI = new Uri(mss_format).ToString();
                sigReq.MSS_Format = mfObject;
            }

            sigReq.MessagingMode = messagingMode;
            sigReq.AdditionalServices = new AdditionalServiceType[] {
                new AdditionalServiceType()
                {
                    Description = new mssURIType()
                    {
                        mssURI = "http://mobile-id.vn/MSSP/v1.0.0#signingCertificate"
                    }
                },
                new AdditionalServiceType()
                {
                    Description = new mssURIType()
                    {
                        mssURI = "http://mobile-id.vn/MSSP/v1.0.0#signatureValidation"
                    }
                }
            };
            sigReq.TimeOut = "300";

            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12 | SecurityProtocolType.Ssl3;
            ServicePointManager.Expect100Continue = false;
            ServicePointManager.ServerCertificateValidationCallback += new RemoteCertificateValidationCallback((sender, certificate, chain, policyErrors) => { return true; });
            
            MSS_SignatureBinding mssSignatureService = new MSS_SignatureBinding();
            mssSignatureService.Url = msspSignatureUrl;
            mssSignatureService.UseDefaultCredentials = true;
            mssSignatureService.ClientCertificates.Add(apCertificate);

            MSS_SignatureRespType sigResp = mssSignatureService.MSS_Signature(sigReq);

            long statusCode = long.Parse(sigResp.Status.StatusCode.Value);

            byte[] sigBuff = sigResp.MSS_Signature.Item;

            return sigBuff;
        }

        public virtual byte[] GetCert(string phone)
        {
            String apTransId = "A" + DateTime.Now.ToString("yyMMddHH") + DateTime.Now.Minute / 15;
            String msisdn = phone;

            //String signatureProfile = "http://mobile-id.vn/digitalSignature";
            //MessagingModeType messagingMode = MessagingModeType.synch;

            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12 | SecurityProtocolType.Ssl3;
            ServicePointManager.Expect100Continue = false;
            ServicePointManager.ServerCertificateValidationCallback += new RemoteCertificateValidationCallback((sender, certificate, chain, policyErrors) => { return true; });
            
            MessageAbstractTypeAP_Info aiObject = new MessageAbstractTypeAP_Info();
            aiObject.AP_ID = apId;
            aiObject.AP_PWD = apPwd;

            aiObject.AP_TransID = apTransId;
            aiObject.Instant = DateTime.Now;

            MSS_RegistrationReqType reg = new MSS_RegistrationReqType();
            reg.MajorVersion = BigInteger.ValueOf(1L).ToString();
            reg.MinorVersion = BigInteger.ValueOf(1L).ToString();
            reg.AP_Info = aiObject;

            MessageAbstractTypeMSSP_Info miObject = new MessageAbstractTypeMSSP_Info();
            MeshMemberType _msspId = new MeshMemberType();
            _msspId.URI = msspId;
            miObject.MSSP_ID = _msspId;
            reg.MSSP_Info = miObject;

            MobileUserType muObject = new MobileUserType();
            muObject.MSISDN = msisdn;
            reg.MobileUser = muObject;

            MSS_RegistrationBinding mssRegService = new MSS_RegistrationBinding();
            mssRegService.Url = msspRegUrl;
            mssRegService.UseDefaultCredentials = true;
            mssRegService.ClientCertificates.Add(apCertificate);

            MSS_RegistrationRespType regResp = mssRegService.MSS_Registration(reg);
            long statusCode = long.Parse(regResp.Status.StatusCode.Value);

            CertificateResponse[] certResp = regResp.CertificateResponse;
            //foreach(var cert in certResp)
            //{
            //    X509Certificate2 cer = new X509Certificate2(cert.X509Certificate);
            //    Console.WriteLine(cer.Subject);
            //}
            return certResp[0].X509Certificate;
        }
    }
}
